<?php

namespace Project\Pm\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Project\Pm\Http\Requests\PageRequest;
use Project\Pm\Interfaces\PageRepositoryInterface;
use Project\Pm\Models\Page;

/**
 * Resource controller class for page.
 */
class PageResourceController extends BaseController
{

    /**
     * Initialize page resource controller.
     *
     * @param type PageRepositoryInterface $page
     *
     * @return null
     */
    public function __construct(PageRepositoryInterface $page)
    {
        parent::__construct();
        $this->repository = $page;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Project\Pm\Repositories\Criteria\PageResourceCriteria::class);
    }

    /**
     * Display a list of page.
     *
     * @return Response
     */
    public function index(PageRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Project\Pm\Repositories\Presenter\PagePresenter::class)
                ->$function();
        }

        $pages = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('pm::page.names'))
            ->view('pm::page.index', true)
            ->data(compact('pages', 'view'))
            ->output();
    }

    /**
     * Display page.
     *
     * @param Request $request
     * @param Model   $page
     *
     * @return Response
     */
    public function show(PageRequest $request, Page $page)
    {

        if ($page->exists) {
            $view = 'pm::page.show';
        } else {
            $view = 'pm::page.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('pm::page.name'))
            ->data(compact('page'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new page.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PageRequest $request)
    {

        $page = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('pm::page.name')) 
            ->view('pm::page.create', true) 
            ->data(compact('page'))
            ->output();
    }

    /**
     * Create new page.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PageRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $page                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('pm::page.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('pm/page/' . $page->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/pm/page'))
                ->redirect();
        }

    }

    /**
     * Show page for editing.
     *
     * @param Request $request
     * @param Model   $page
     *
     * @return Response
     */
    public function edit(PageRequest $request, Page $page)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('pm::page.name'))
            ->view('pm::page.edit', true)
            ->data(compact('page'))
            ->output();
    }

    /**
     * Update the page.
     *
     * @param Request $request
     * @param Model   $page
     *
     * @return Response
     */
    public function update(PageRequest $request, Page $page)
    {
        try {
            $attributes = $request->all();

            $page->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('pm::page.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('pm/page/' . $page->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('pm/page/' . $page->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the page.
     *
     * @param Model   $page
     *
     * @return Response
     */
    public function destroy(PageRequest $request, Page $page)
    {
        try {

            $page->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('pm::page.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('pm/page/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('pm/page/' . $page->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple page.
     *
     * @param Model   $page
     *
     * @return Response
     */
    public function delete(PageRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('pm::page.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('pm/page'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/pm/page'))
                ->redirect();
        }

    }

    /**
     * Restore deleted pages.
     *
     * @param Model   $page
     *
     * @return Response
     */
    public function restore(PageRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('pm::page.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/pm/page'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/pm/page/'))
                ->redirect();
        }

    }

}
