<?php

namespace Project\Pm\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Project\Pm\Http\Requests\RoadmapRequest;
use Project\Pm\Interfaces\RoadmapRepositoryInterface;
use Project\Pm\Models\Roadmap;

/**
 * Resource controller class for roadmap.
 */
class RoadmapResourceController extends BaseController
{

    /**
     * Initialize roadmap resource controller.
     *
     * @param type RoadmapRepositoryInterface $roadmap
     *
     * @return null
     */
    public function __construct(RoadmapRepositoryInterface $roadmap)
    {
        parent::__construct();
        $this->repository = $roadmap;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Project\Pm\Repositories\Criteria\RoadmapResourceCriteria::class);
    }

    /**
     * Display a list of roadmap.
     *
     * @return Response
     */
    public function index(RoadmapRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Project\Pm\Repositories\Presenter\RoadmapPresenter::class)
                ->$function();
        }

        $roadmaps = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('pm::roadmap.names'))
            ->view('pm::roadmap.index', true)
            ->data(compact('roadmaps', 'view'))
            ->output();
    }

    /**
     * Display roadmap.
     *
     * @param Request $request
     * @param Model   $roadmap
     *
     * @return Response
     */
    public function show(RoadmapRequest $request, Roadmap $roadmap)
    {

        if ($roadmap->exists) {
            $view = 'pm::roadmap.show';
        } else {
            $view = 'pm::roadmap.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('pm::roadmap.name'))
            ->data(compact('roadmap'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new roadmap.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RoadmapRequest $request)
    {

        $roadmap = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('pm::roadmap.name')) 
            ->view('pm::roadmap.create', true) 
            ->data(compact('roadmap'))
            ->output();
    }

    /**
     * Create new roadmap.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RoadmapRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $roadmap                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('pm::roadmap.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('pm/roadmap/' . $roadmap->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/pm/roadmap'))
                ->redirect();
        }

    }

    /**
     * Show roadmap for editing.
     *
     * @param Request $request
     * @param Model   $roadmap
     *
     * @return Response
     */
    public function edit(RoadmapRequest $request, Roadmap $roadmap)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('pm::roadmap.name'))
            ->view('pm::roadmap.edit', true)
            ->data(compact('roadmap'))
            ->output();
    }

    /**
     * Update the roadmap.
     *
     * @param Request $request
     * @param Model   $roadmap
     *
     * @return Response
     */
    public function update(RoadmapRequest $request, Roadmap $roadmap)
    {
        try {
            $attributes = $request->all();

            $roadmap->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('pm::roadmap.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('pm/roadmap/' . $roadmap->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('pm/roadmap/' . $roadmap->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the roadmap.
     *
     * @param Model   $roadmap
     *
     * @return Response
     */
    public function destroy(RoadmapRequest $request, Roadmap $roadmap)
    {
        try {

            $roadmap->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('pm::roadmap.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('pm/roadmap/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('pm/roadmap/' . $roadmap->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple roadmap.
     *
     * @param Model   $roadmap
     *
     * @return Response
     */
    public function delete(RoadmapRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('pm::roadmap.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('pm/roadmap'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/pm/roadmap'))
                ->redirect();
        }

    }

    /**
     * Restore deleted roadmaps.
     *
     * @param Model   $roadmap
     *
     * @return Response
     */
    public function restore(RoadmapRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('pm::roadmap.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/pm/roadmap'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/pm/roadmap/'))
                ->redirect();
        }

    }

}
